#include "Logging.h"
#include "Call.h"
#include <stdio.h>

#include "resip/dum/InviteSession.hxx"
#include "resip/dum/ClientInviteSession.hxx"
#include "Agent.h"

using namespace resip;
using namespace Dumais::JSON;

Call::Call(resip::DialogUsageManager &dum):AppDialogSet(dum)
{
    this->mRtpSession = 0;
    this->mIncomming = false;
    this->mQueuedCall = 0;
    this->mAssignedCall = 0;
}

Call::~Call()
{
    if (this->mRtpSession)
    {
        delete this->mRtpSession;
        this->mRtpSession = 0;
    }
}

void Call::setIncomming(bool i)
{
    this->mIncomming = i;
}

void Call::addRTPObserver(RTPObserver *obs)
{
    this->mRtpObservers.push_back(obs);
}


void Call::setRTPSession(RTPSession *rtpSession)
{
    this->mRtpSession = rtpSession;
    this->mRtpSession->addObserver(this); //TODO should unsubscribe if overwriting observer
}

RTPSession* Call::getRTPSession()
{
    return this->mRtpSession;
}

void Call::onSoundQueueEmpty()
{
    for (std::list<RTPObserver*>::iterator it = this->mRtpObservers.begin();it!=this->mRtpObservers.end();it++)
    {
        (*it)->onRTPSessionSoundQueueEmpty(this);
    }

}

DialogUsageManager& Call::getDUM()
{
    return this->mDum;
}

bool Call::isIncomming()
{
    return this->mIncomming;
}

std::string Call::getDigitQueue()
{
    return this->mDigitQueue;
}

void Call::clearDigitQueue()
{
    this->mDigitQueue="";
}

resip::NameAddr Call::getFrom()
{
    return mFrom;
}

resip::NameAddr Call::getContact()
{
    return mContact;
}

const char* Call::getOwner()
{
    return mOwner.uri().user().data();

}

Call* Call::getAssociatedQueuedCall()
{
    return mQueuedCall;
}


std::string Call::getID()
{
    return this->getDialogSetId().getCallId().c_str();
}

void Call::onCallQueued()
{
    mCallState = Queued;
}

void Call::onAssigned()
{
    mCallState = Assigned;
}

void Call::assignCall(Call *call, Agent *agent)
{
    //TODO: if we load config dynamically, this could be dangerous
    mAgent = agent;
    mAgent->onAssigning();
    call->onAssigned();
    mQueuedCall = call;
    call->mAssignedCall = this;
    mCallState = CallingAgent;
}

void Call::onAssignFailed(bool logout)
{
    if (mCallState==CallingAgent)
    {
        if (logout) mAgent->onAssignFailed();
        mQueuedCall->onAssignFailed();
        
    } else if (mCallState == Assigned)
    {
        mCallState = Queued;
    }
}



void Call::onTerminated()
{
    // If the queued call terminates while it is assigned, cancel the second call.

    if (this->mCallState==Call::Assigned) // Assigned queued call
    {
        if (mAssignedCall)
        {
            mAssignedCall->end();
        }
    }

    mCallState = Terminated;
    
}

Call::CallState Call::getCallState()
{
    return this->mCallState;
}

void Call::toJSON(Dumais::JSON::JSON &json)
{
    JSON j = json.addObject();
    j.addValue(getID(),"id");

    std::string s;
    switch (mCallState)
    {
        case Queued:
            s= "queued";
            break;
        case Assigned:
            s= "assigned";
            break;
        case CallingAgent:
            s= "callingagent";
            break;
        case Terminated:
            s= "terminated";
            break;
    }
    j.addValue(s,"state");
    j.addValue(mFrom.uri().getAor().data(),"from");
    j.addValue( mTo.uri().getAor().data(),"to");
}

